@@ -107,15 +107,13 @@ addtoChooseSummary <- function(choose.summary, term, DF = NA, denDF = NA, p = NA
     stop("Supplied data.frame does not contain all terms in terms.marginality")
 
   #Process DF argument
-#  attr(object, which = "omit.df") <- FALSE
   object <- setdf(object, df = DF, which.df = "DF")
 
   #Process denDF argument
   object <- setdf(object, df = denDF, which.df = "denDF")
   DF <- attr(object, which = "DF")
   denDF <- attr(object, which = "denDF")
-#  omit.df <- attr(object, which = "omit.df")
-  
+
   #perform tests
   sig.terms <- vector("list", length = 0)
   noterms <- dim(terms.marginality)[1]
@@ -191,7 +189,8 @@ addtoChooseSummary <- function(choose.summary, term, DF = NA, denDF = NA, p = NA
 }
 
 "chooseModel.asrtests" <- function(object, terms.marginality=NULL, alpha = 0.05, 
-                                   allow.unconverged = TRUE, checkboundaryonly = FALSE, 
+                                   allow.unconverged = TRUE, allow.fixedcorrelation = TRUE, 
+                                   checkboundaryonly = FALSE, 
                                    drop.ran.ns=TRUE, positive.zero = FALSE, 
                                    bound.test.parameters = "none", 
                                    drop.fix.ns=FALSE, denDF = "numeric",  dDF.na = "none", 
@@ -256,6 +255,7 @@ addtoChooseSummary <- function(choose.summary, term, DF = NA, denDF = NA, p = NA
     term <- (rownames(terms.marginality))[j]
     current.asrt <- testranfix.asrtests(asrtests.obj = current.asrt, term, 
                                         alpha=alpha, allow.unconverged = allow.unconverged, 
+                                        allow.fixedcorrelation = allow.fixedcorrelation, 
                                         checkboundaryonly = checkboundaryonly,  
                                         drop.ran.ns = drop.ran.ns, 
                                         positive.zero = positive.zero, 
@@ -308,7 +308,8 @@ addtoChooseSummary <- function(choose.summary, term, DF = NA, denDF = NA, p = NA
                                        dropFixed = NULL, addFixed = NULL, 
                                        dropRandom = NULL,  addRandom = NULL, 
                                        newResidual = NULL, label = "Changed terms", 
-                                       allow.unconverged = TRUE, checkboundaryonly = FALSE, 
+                                       allow.unconverged = TRUE, allow.fixedcorrelation = TRUE,
+                                       checkboundaryonly = FALSE, 
                                        trace = FALSE, update = TRUE, denDF = "numeric", 
                                        set.terms = NULL, ignore.suffices = TRUE, 
                                        bounds = "P", initial.values = NA, 
@@ -327,6 +328,22 @@ addtoChooseSummary <- function(choose.summary, term, DF = NA, denDF = NA, p = NA
   options <- c("AIC", "BIC") #, "both")
   ic.type <- options[check.arg.values(which.IC, options)]
   
+  #Check for fixed correlations in supplied asrtests.obj
+  if (!isFixedCorrelOK(asrtests.obj$asreml.obj, allow.fixedcorrelation = allow.fixedcorrelation))
+  {
+    if ("formulae" %in% names(asrtests.obj$asreml.obj))
+      kresp <- asrtests.obj$asreml.obj$formulae$fixed[[2]]
+    else
+    {
+      if ("fixed.formula" %in% names(asrtests.obj$asreml.obj))
+        kresp <- asrtests.obj$asreml.obj$fixed.formula[[2]]
+      else
+        kresp <- NULL
+    }
+    warning(paste("The estimated value of one or more correlations in the supplied asreml fit for", kresp,
+                  "is fixed and allow.fixedcorrelation is FALSE"))
+  }
+  
   #Calculate the IC for the incoming fit
   old.IC <- infoCriteria(asrtests.obj$asreml.obj, IClikelihood = ic.lik, 
                          bound.exclusions = bound.exclusions, 
@@ -381,7 +398,7 @@ addtoChooseSummary <- function(choose.summary, term, DF = NA, denDF = NA, p = NA
                                     dropFixed = dropFixed, addFixed = addFixed, 
                                     dropRandom = dropRandom,  addRandom = addRandom, 
                                     newResidual = newResidual, label = label, 
-                                    allow.unconverged = TRUE, 
+                                    allow.unconverged = TRUE, allow.fixedcorrelation = TRUE, 
                                     checkboundaryonly = checkboundaryonly, 
                                     trace = trace, update = update, denDF = denDF, 
                                     set.terms = set.terms, ignore.suffices = ignore.suffices, 
@@ -424,29 +441,37 @@ addtoChooseSummary <- function(choose.summary, term, DF = NA, denDF = NA, p = NA
       }
     } else
     {
-      #Make the comparison
-      action <- "Unswapped"
-      if ((ic.type == "AIC" & diff.IC["AIC"] < 0) || 
-          (ic.type == "BIC" & diff.IC["BIC"] < 0))
+      #Check fixed correlation
+      if (!isFixedCorrelOK(new.asreml.obj, allow.fixedcorrelation = allow.fixedcorrelation))
       {
-        change <- TRUE
-        action <- "Swapped"
-      }
-      
-      #check convergence, when it is allowed
-      if (allow.unconverged)
+        action <- "Unchanged - fixed correlation"
+        change <- FALSE
+      } else
       {
-        if (!asreml.obj$converge && !new.asreml.obj$converge)
+        #Make the comparison
+        action <- "Unswapped"
+        if ((ic.type == "AIC" & diff.IC["AIC"] < 0) || 
+            (ic.type == "BIC" & diff.IC["BIC"] < 0))
         {
-          action <- paste(action, " - both unconverged", sep="")
-        } else
+          change <- TRUE
+          action <- "Swapped"
+        }
+        
+        #check convergence, when it is allowed
+        if (allow.unconverged)
         {
-          if (!asreml.obj$converge)
-            action <- paste(action, " - old unconverged", sep="")
-          else
+          if (!asreml.obj$converge && !new.asreml.obj$converge)
           {
-            if (!new.asreml.obj$converge)
-              action <- paste(action, " - new unconverged", sep="")
+            action <- paste(action, " - both unconverged", sep="")
+          } else
+          {
+            if (!asreml.obj$converge)
+              action <- paste(action, " - old unconverged", sep="")
+            else
+            {
+              if (!new.asreml.obj$converge)
+                action <- paste(action, " - new unconverged", sep="")
+            }
           }
         }
       }
