@@ -254,6 +254,10 @@ test_that("IC_wheat94_asreml4", {
   if (getTestPvalue(current.asrt, label = "Col") > 0.05)
     current.asrt <- testranfix(current.asrt, term = "spl(Col)", alpha = 0.20)
   
+  #tests for getTestPvalue
+  testthat::expect_true(abs(getTestPvalue(current.asrt, label = "Col") - .5944761) < 1e-05)
+  testthat::expect_error(getTestPvalue(current.asrt, label = "Co"))
+  
   #Test units term
   current.asrt <- testranfix(current.asrt, term = "units", alpha = 0.20)
   testthat::expect_equal(nrow(current.asrt$test.summary), 10)
