@@ -28,7 +28,7 @@ test_that("sort.alldiffs4", {
   testthat::expect_equal(nrow(diffs$predictions),120)
   testthat::expect_equal(ncol(diffs$predictions),8)
   testthat::expect_equal(as.character(diffs$predictions$Genotype[1]),"Axe")
-  testthat::expect_equal(length(attributes(diffs)),8)
+  testthat::expect_equal(length(attributes(diffs)),11)
   testthat::expect_true(is.null(attr(diffs, which = "sortOrder")))
   
   testthat::expect_silent(plotPredictions(data = diffs$predictions, 
@@ -54,7 +54,7 @@ test_that("sort.alldiffs4", {
   testthat::expect_equal(nrow(diffs.sort$predictions),120)
   testthat::expect_equal(ncol(diffs.sort$predictions),8)
   testthat::expect_equal(as.character(diffs.sort$predictions$Genotype[1]),"Gladius")
-  testthat::expect_equal(length(attributes(diffs.sort)),10)
+  testthat::expect_equal(length(attributes(diffs.sort)),13)
   testthat::expect_equal(length(attr(diffs.sort, which = "sortOrder")),10)
   
   #Test sort.alldiffs with supplied sortOrder
@@ -73,12 +73,13 @@ test_that("sort.alldiffs4", {
   testthat::expect_equal(attr(diffs.sort, which = "sortOrder"),
                          attr(diffs2.sort, which = "sortOrder"))
   
-  #Test sort.alldiffs with sortWithinVals and increasing order
+  #Test sort.alldiffs with sortParallelToCombo and increasing order
   diffs1.sort <- sort(diffs, sortFactor = "Genotype", 
-                      sortWithinVals = list(A = "N3", B = "D4"),
+                      sortParallelToCombo = list(A = "N3", B = "D4"),
                       decreasing = TRUE)
+  diffs1.sort <- renewClassify(diffs1.sort, newclassify = "A:B:Genotype")
   testthat::expect_is(diffs1.sort, "alldiffs")
-  testthat::expect_equal(as.character(diffs1.sort$predictions$Genotype[2]),"Wyalkatchem")
+  testthat::expect_equal(as.character(attr(diffs1.sort, which = "sortOrder")[2]),"Wyalkatchem")
   testthat::expect_equal(length(attr(diffs1.sort, which = "sortOrder")), 10)
   #Check plot
   testthat::expect_silent(plotPredictions(data = diffs1.sort$predictions, 
@@ -90,6 +91,63 @@ test_that("sort.alldiffs4", {
                                           sortFactor = "Genotype"))
 })
 
+
+cat("#### Test for sort.predictions.frame on WaterRunoff with asreml4\n")
+test_that("sort.predictions.frame4", {
+  skip_if_not_installed("asreml")
+  skip_on_cran()
+  library(asreml)
+  library(asremlPlus)
+  library(dae)
+  data(WaterRunoff.dat)
+  tmp <- subset(WaterRunoff.dat, Date == "05-18")
+  
+  ##Use asreml to get predictions
+  #Analyse pH  
+  m1.asr <- do.call(asreml, 
+                    list(fixed = pH ~ Benches + (Sources * (Type + Species)), 
+                         random = ~ Benches:MainPlots,
+                         keep.order=TRUE, data= tmp))
+  current.asrt <- as.asrtests(m1.asr, NULL, NULL)
+  current.asrt <- as.asrtests(m1.asr)
+  current.asrt <- rmboundary(current.asrt)
+  m1.asr <- current.asrt$asreml.obj
+  
+  #Get predictions and associated statistics  
+  TS.diffs <- predictPlus.asreml(classify = "Sources:Type", 
+                                 asreml.obj = m1.asr, tables = "none", 
+                                 wald.tab = current.asrt$wald.tab, 
+                                 present = c("Type","Species","Sources"))
+  
+  #Use sort.predictions.frame and save order for use with other response variables
+  TS.preds <- TS.diffs$predictions
+  TS.preds.sort <- sort(TS.preds, classify = "Sources:Type", 
+                        sortFactor = "Sources", sortParallelToCombo = list(Type = "Control"))
+  TS.preds.sort[TS.preds.sort$Type == "Control", ]
+  sort.order <- attr(TS.preds.sort, which = "sortOrder")
+  testthat::expect_true(all(sort.order == c("Rain+Basalt", "Rain+Dolomite", "Tap water", 
+                                            "Recycled water", "Rainwater", "Rain+Quartzite")))
+  
+  diffs.full <- predictPlus.asreml(asreml.obj = m1.asr, 
+                                   classify = "Sources:Type:Species", 
+                                   wald.tab = current.asrt$wald.tab, 
+                                   present = c("Type","Species","Sources"),
+                                   tables = "none", Vmatrix = TRUE)
+  testthat::expect_true(setequal(names(diffs.full$predictions), 
+                                 c("Sources", "Type", "Species", "predicted.value", 
+                                   "standard.error", "upper.Confidence.limit", 
+                                   "lower.Confidence.limit", "est.status")))
+  pred <- diffs.full$predictions
+  diffs.sort <- sort(diffs.full, sortFactor = "Species", sortNestingFactor = "Type",
+                    sortParallelToCombo = list(Sources = "Rain+Dolomite"))
+ testthat::expect_true(all(diffs.sort$predictions$Species[diffs.sort$predictions$Sources ==  "Rain+Dolomite"] == 
+                             c("S. oymwjrcnepv", "S. iqscjbogxah", "S. ocphawvtlgi", "S. orcxszbujml", 
+                               "S. hbpgtylxqku", "S. tkujbvipoyr", "S. xeqackngdrt", 'Non-planted')))
+ testthat::expect_false(all(diffs.sort$predictions$predicted.value == pred$predicted.value))
+ t <- merge(diffs.sort$predictions, pred, by = c("Sources","Type","Species"))
+ testthat::expect_true(all(t$predicted.value.x == t$predicted.value.y))
+})
+  
 cat("#### Test for sort.alldiffs on Oats with asreml4\n")
 test_that("sort.alldiffs4", {
   skip_if_not_installed("asreml")
@@ -112,7 +170,7 @@ test_that("sort.alldiffs4", {
   Var.diffs <- as.alldiffs(predictions = Var.pred$pvals, 
                            sed = Var.pred$sed, 
                            tdf = den.df)
-  testthat::expect_equal(length(attributes(Var.diffs)),3)
+  testthat::expect_equal(length(attributes(Var.diffs)),4)
   testthat::expect_true(is.null(attr(Var.diffs, which = "sortOrder")))
   
   #Test for predictPlus no sorting
@@ -123,7 +181,7 @@ test_that("sort.alldiffs4", {
   testthat::expect_equal(nrow(diffs$predictions),12)
   testthat::expect_equal(ncol(diffs$predictions),7)
   testthat::expect_equal(as.character(diffs$predictions$Variety[1]),"Victory")
-  testthat::expect_equal(length(attributes(diffs)),8)
+  testthat::expect_equal(length(attributes(diffs)),11)
   testthat::expect_true(is.null(attr(diffs, which = "sortOrder")))
   
   testthat::expect_silent(plotPredictions(data = diffs$predictions, 
@@ -139,8 +197,18 @@ test_that("sort.alldiffs4", {
   #Test for sort.alldiffs
   diffs.sort <- sort(diffs, sortFactor = "Variety", decreasing = TRUE)
   testthat::expect_equal(as.character(diffs.sort$predictions$Variety[1]),"Marvellous")
+  testthat::expect_equal(rownames(diffs.sort$differences)[1],"0,Marvellous")
+  testthat::expect_equal(colnames(diffs.sort$p.differences)[1],"0,Marvellous")
   testthat::expect_silent(plotPvalues(diffs.sort, gridspacing = 3))
   
+  diffs.LSD <- redoErrorIntervals(diffs, error.intervals = "halfLeast", LSDtype = "factor", LSDby = "Variety")
+  diffs.LSD.sort <- sort(diffs.LSD, sortFactor = "Variety", decreasing = TRUE)
+  
+  #Test for sort.predictions.frame
+  preds <- diffs$predictions
+  preds.sort <- sort.predictions.frame(preds, classify = "Nitrogen:Variety", sortFactor = "Variety", decreasing = TRUE)
+  testthat::expect_equal(as.character(preds.sort$Variety[1]),"Marvellous")
+
   #Test for predictPresent
   mx.asr <- asreml(Yield ~ xNitrogen*Variety, 
                    random=~Blocks/Wplots,
@@ -160,7 +228,7 @@ test_that("sort.alldiffs4", {
   testthat::expect_equal(nrow(diffs[[1]]$predictions),12)
   testthat::expect_equal(ncol(diffs[[1]]$predictions),7)
   testthat::expect_equal(as.character(diffs[[1]]$predictions$Variety[[1]]),"Marvellous")
-  testthat::expect_equal(length(attributes(diffs$xNitrogen.Variety)),10)
+  testthat::expect_equal(length(attributes(diffs$xNitrogen.Variety)),13)
   testthat::expect_equal(length(attr(diffs[[1]], which = "sortOrder")),3)
   
   #Test for predictPlus with sortFactor
@@ -174,7 +242,7 @@ test_that("sort.alldiffs4", {
   testthat::expect_equal(nrow(diffs$predictions),12)
   testthat::expect_equal(ncol(diffs$predictions),7)
   testthat::expect_equal(as.character(diffs$predictions$Variety[1]),"Marvellous")
-  testthat::expect_equal(length(attributes(diffs)),10)
+  testthat::expect_equal(length(attributes(diffs)),13)
   testthat::expect_true(all(attr(diffs, which = "sortOrder") == 
                               levels(diffs$predictions$Variety)))
   testthat::expect_true(all(attr(diffs, which = "sortOrder") == 
@@ -214,7 +282,7 @@ test_that("classify.sort4", {
   Var.diffs <- as.alldiffs(predictions = Var.pred$pvals, 
                            sed = Var.pred$sed, 
                            tdf = den.df)
-  testthat::expect_equal(length(attributes(Var.diffs)),3)
+  testthat::expect_equal(length(attributes(Var.diffs)),4)
   testthat::expect_true(is.null(attr(Var.diffs, which = "sortOrder")))
   testthat::expect_true(all(names(Var.diffs$predictions)[1:2] ==  c("Variety","Nitrogen")))
   
